#!/opt/VRTSperl/bin/perl -w
#------------------------------------------------------------------
# $Copyright: Copyright (c) 2023 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#
# $Header: /project/isv-agent/VProAgents/TibcoEMS/UpdateTypes.pl ,v 1.10 2023/05/29 08:20:47 mrampura Exp $
#
# $sAuthor: $
#
# $Date: 2023/05/29 08:20:47 $
# $Revision: 1.10 $
#------------------------------------------------------------------

use strict;
use POSIX;
use Getopt::Std;
use Fcntl;
use File::Path qw ( mkpath );
use File::Copy;


use constant HASYS     => "/opt/VRTSvcs/bin/hasys";
use constant HARES     => "/opt/VRTSvcs/bin/hares";
use constant HATYPE    => "/opt/VRTSvcs/bin/hatype";
use constant HACONF    => "/opt/VRTSvcs/bin/haconf";
use constant HACF      => "/opt/VRTSvcs/bin/hacf";
use constant HACLUS    => "/opt/VRTSvcs/bin/haclus";
use constant HAATTR    => "/opt/VRTSvcs/bin/haattr";
use constant MAINCFLOC => "/etc/VRTSvcs/conf/config";
use constant VCSLOGDIR => "/var/VRTSvcs/log";

sub usage {
  warn << "EOF";
  Update:- Update the Types.cf.
  Usage:
  $0 
  -h  --help      Help (this message)
EOF
  exit 1;
}

my $sType="TibcoEMS";
my ( $sCommand, $sCmdOut, $iExitCode, $sMainCmdFile, $sAttribute, $sRestoreMsg, $sOptStr, 
     $sTypeFile, $sLine, $sLogDir, $sOldArgList, $sNewArgList, $sKey, 
     $bVCSStatus, @lsOldArgList, @lsNewArgList, %hsOpts, $delayCount, $maxDelay ) = ();

my %hsSupported = ( '--help'    => '-h' );
my %hsOverriddenResAttr = ();

#-----------------------
# Collect command line..
#-----------------------
$sOptStr = 'h';

grep {                      # Filter..
  $hsSupported { $_ }    && # Defined..
  s/$_/$hsSupported{$_}/ || # Then substitute..
  1                         # Else, continue..
} @ARGV;                    # From what's been passed..

&getopts ( $sOptStr, \%hsOpts ) || &usage ();

#------------
# Show Usage 
#------------
if ( $hsOpts { h } ) {
  &usage();
}


#-------------------------------------------
# Remove the Log Dir, If not already present
#-------------------------------------------
$sLogDir = VCSLOGDIR . "/Update${sType}Type_$$";

if ( -e $sLogDir ) {
  $sCommand = "/bin/rm -rf $sLogDir > /dev/null 2>&1";
  system ( $sCommand );
  if( $iExitCode = $? >> 8 ) {
    warn "Failed to remove existing log Directory [$sLogDir]\n";
    exit 1;
  }  
}

#-------------------
# Create the Log Dir
#-------------------
if ( ! mkpath ( $sLogDir, 0, 0700 ) ) {
  warn "Failed to make log directory [$sLogDir]: $!\n";
  exit 1;   
}

#--------------------------------------
# Open the log file for printing output
#--------------------------------------
sysopen ( LOG, "$sLogDir/${sType}_update.log",
          O_WRONLY|O_SYNC|O_CREAT|O_TRUNC, 0640 ) ||
die "Failed to create and open file  [$sLogDir/${sType}_update.log]: $!\n";

#------------------------
# Create Log Header 
#------------------------
my @lsOsDetails = uname ();
my $sTime = strftime '%Y/%m/%d %H:%M:%S', localtime; 

my $sHeader = "#
# Log Name:     $sLogDir/${sType}.log
# System:       $lsOsDetails[1]
# SysInfo:      $lsOsDetails[0]:$lsOsDetails[1],$lsOsDetails[3],$lsOsDetails[2],$lsOsDetails[-1]
# Created:      $sTime
# \n\n";
$sHeader =~ s/SunOS/Solaris/g;

print LOG $sHeader;

#-------------------------------
# Check if VCS is running or not 
#-------------------------------
$bVCSStatus = &getVCSStatus();

#-----------------------------------------
# Proceed only if new package is installed
#-----------------------------------------
if ( -e ( $sTypeFile = "/etc/VRTSagents/ha/conf/${sType}/${sType}Types.cf"   ) ||  # For Linux,HPUX,AIX
     -e ( $sTypeFile = "/etc/VRTSagents/ha/conf/${sType}/${sType}Types51.cf" ) ||  # For Solaris
     -e ( $sTypeFile = "/etc/VRTSagents/ha/conf/${sType}/${sType}Types50.cf" ) ||
     -e ( $sTypeFile = "/etc/VRTSvcs/conf/sample_${sType}/${sType}Types.cf" ) ) {  
  if ( ! open ( TYPEFILDES, $sTypeFile ) ) {
    printError("Failed to open Agent type file [$sTypeFile] for read");
    close ( LOG );
    exit 1;
  }
  else {
    print LOG "[Success] Agent Type File[$sTypeFile] is opened for read\n";
  }
}
else {
	print LOG "[Failed] $sType Agent is not installed.\n";
	print "For more details check the log file [$sLogDir/${sType}_update.log]\n";
	close ( LOG ) || die "Failed to close [$sLogDir/${sType}_update.log] after read: $!\n";
	exit 0;
}

#---------------------------
# Take the backup of main.cf
#---------------------------
$sCommand = "/bin/cp -f /etc/VRTSvcs/conf/config/main.cf $sLogDir/main.cf.before_update > /dev/null 2>&1";
system ( $sCommand );

if ( $iExitCode = $? >> 8 ) {
  printError ("Failed to backup the current main.cf");
  close ( LOG );
  exit 1;
}
else {
  print LOG "[Success] The backup of main.cf before update is ". 
            "present at [$sLogDir/main.cf.before_update]\n";
}

$sRestoreMsg = "\n\nThe backup of main.cf before the execution of this update script is available 
at [$sLogDir/main.cf.before_update]. You can restore your configuration using this file.\n";


if ( $bVCSStatus ){			# If VCS is Running.... 
  print LOG "VCS is currently running.\n";

  #-------------------------------
  # Dump the current configuration
  #-------------------------------
  $delayCount = 0;
  $maxDelay = 10; 
  while ( $delayCount < $maxDelay ) {
    $sCommand = HACONF . " -dump ";
    $sCmdOut = `$sCommand 2>&1`;
    if ( $iExitCode = $? >> 8 ) {
      if ( $sCmdOut =~ m/Dump already in progress/ ) {
        sleep 5;
      }
    }
    else {
      print LOG "[Success] Changed configuration is dumped in main.cf\n";
      last;	
    }
    $delayCount = $delayCount + 1;
  }
  if ( $delayCount == $maxDelay ) {
    printError("Failed to dump the changed configuration in main.cf");
    printError("[ ERROR ] $sCmdOut");
    close ( LOG );
    exit 1;
  }

  #--------------------------
  # Check if Type is imported
  #--------------------------
  $sCommand =  HATYPE . " -display $sType";
  $sCmdOut = `$sCommand 2>&1`;
  if ( $iExitCode = $? >> 8 ) {
    if ( $sCmdOut =~ /Type does not exist/ ) {
      print LOG "[Info] The Agent Type [$sType] is not imported in the cluster.\nUse [/etc/VRTSagents/ha/conf/$sType/${sType}Types.cf] to import the Agent Type";
      close ( LOG );
      exit 0;
    } else {
      print LOG "Check whether VCS is running on the system. If not, start VCS and try running ${sType}_update.pl script again\n";
      printError("Not able to execute [hatype] command on this node");
      close ( LOG );
      exit 1;
    }
  }  
  else {
    print LOG "[Success] Type [$sType] exists check passed\n";
  } 

  #-----------------------------------------
  # Get the old ArgList from Type definition
  #-----------------------------------------

  $sCommand = HATYPE . " -value $sType ArgList";
  $sCmdOut = `$sCommand 2>&1`;
  if ( $iExitCode = $? >> 8 ) {
    printError ("Not able to get the old ArgList");
    close ( LOG );
    exit 1;
  }
  else {
    print LOG "[Success] Got the old ArgList\n";
    $sOldArgList = $sCmdOut;
  }
  
  @lsOldArgList = ( $sCmdOut =~ /(\S+)/g );

  #---------------------------------------
  # Get the New ArgList from New Types.cf
  #---------------------------------------

  while ( $sLine = <TYPEFILDES> ) {
    if ( $sLine =~ m/.*ArgList.*=\s+\{\s+(.*)\}/ ) {
      $sNewArgList = $1 if $1;
    }
    $sNewArgList =~ s/,/ /g if $sNewArgList;
  }
  @lsNewArgList = ( $sNewArgList  =~ /(\S+)/g );
  close ( TYPEFILDES );


  print LOG "\nold ArgList: ". join (' ',@lsOldArgList);
  print LOG "\nnew ArgList: ". join (' ',@lsNewArgList);

  chomp @lsOldArgList;
  chomp @lsNewArgList;

  #----------------------------------------------------------------------------
  # Get the difference between the two Arglists.
  # Limitation: This will only identify the attributes added in the new ArgList
  #             New Attributes will be added. Old Attributes will not be deleted
  #-----------------------------------------------------------------------------
  my @diff = grep {my $tmp = $_; ! grep($_ eq $tmp, @lsOldArgList)} @lsNewArgList;
  
  if (scalar @diff == 0) {
      print LOG "[Info] No new Attributes found.";
      close ( LOG );
      exit 0;
  }
  my %NewAttrDef = map { $_ => undef }@diff;
  print LOG "\nNew Attributes found:: ". join (' ',@diff) . "\n";

  if ( ! open ( TYPEFILDES, $sTypeFile ) ) {
    printError("Failed to open Agent type file [$sTypeFile] for read");
    close ( LOG );
    exit 1;
  }
  else {
    print LOG "[Success] Agent Type File[$sTypeFile] is opened for read\n";
  
    #-----------------------------------------------------
    # Find the Data type, Attribute name and default value
    # of the new Attributes from the New Types.cf
    # Constrict the command to add this attribute and
    # store in hash
    #-----------------------------------------------------

    while ( $sLine = <TYPEFILDES> ) {
      if ( $sLine =~ m/.*ArgList.*=\s+\{\s+(.*)\}/ ) {
        next;
      }
      if (grep {  $sLine =~ m/\b$_\b/} @diff ) {
        # parse line and store;
        my ($type, $attr, $val) = ();
        ($type,$attr,$val)= $sLine =~ m/(\w+)\s+(\w+)\s*(.*)/g;
        $type =~ s/int/integer/;
        $type =~ s/str/string/;
  
        if (defined $val) {
          $val =~ s/^=\s*//;
          $NewAttrDef{$attr} = HAATTR ." -add $sType $attr -$type $val";
        }
        $NewAttrDef{$attr} = HAATTR ." -add $sType $attr -$type"; 
      }
    }
    close ( TYPEFILDES );
  }


  $sCommand = HACLUS . " -value ReadOnly -localclus";
  $sCmdOut = `$sCommand 2>&1`;
  chomp ( $sCmdOut );
  if ( 1 == $sCmdOut ) {
    $sCommand = HACONF . " -makerw > /dev/null 2>&1";
    system ( $sCommand );
    if ( $iExitCode = $? >> 8 ) {
      printError("Failed to configure the cluster for read and write");
      close ( LOG );
      exit 1;
    }
    else {
      print LOG "[Success] The cluster is configured for read/write\n";
    }
  }

  #-----------------------------------------
  # Add New Attributes using Haattr command.
  #-----------------------------------------
  my $key = ();
  foreach $key (keys %NewAttrDef){
    $sCommand =  $NewAttrDef{$key};
    system ( $sCommand );
    if ( $iExitCode = $? >> 8 ) {
      printError ("Failed to add attribute $key. Manually add the attribute using command [$sCommand]");
    }
    else {
      print LOG "[Success] Added attribute [$key] using commands[$sCommand]\n";
    }
  }


  $sCommand = HATYPE . " -modify $sType ArgList $sNewArgList";
  system ( $sCommand );
  if ( $iExitCode = $? >> 8 ) {
    printError("Failed to modify ArgList. Manually modify the ArgList using command [$sCommand]");
  }
  else {
    print LOG "[Success] Modified ArgList using commands[$sCommand]\n";
  }
  $delayCount = 0;
  $maxDelay = 10; 
  while ( $delayCount < $maxDelay ) {
    $sCommand = HACONF . " -dump ";
    $sCmdOut = `$sCommand 2>&1`;
    if ( $iExitCode = $? >> 8 ) {
      if ( $sCmdOut =~ m/Dump already in progress/ ) {
        sleep 5;
      }
    }
    else {
      print LOG "[Success] Changed configuration is dumped in main.cf\n";
      last;	
    }
    $delayCount = $delayCount + 1;
  }
  if ( $delayCount == $maxDelay ) {
    printError("Failed to dump the changed configuration in main.cf");
    printError("[ ERROR ] $sCmdOut");
    close ( LOG );
    exit 1;
  }
  print "You have successfully added attributes in the agent type [$sType]\n";
  print "For details, see the following log: [$sLogDir/${sType}_update.log]\n";
  close ( LOG );
  exit 0;
}
else {	
     print LOG "VCS is currently not running.\n";
 
  my ( $sOldTypeFile, @lsOldArgList, @lsNewArgList, 
       @lsDepAttr, @lsNewAttr ) = ();
 
  my $sDestTypeFile = MAINCFLOC . "/${sType}Types.cf";
  if ( $^O eq 'solaris' ){
    # For VCS 5.1 or later.
    $sDestTypeFile = MAINCFLOC . "/${sType}Types51.cf";
    if(! -e $sOldTypeFile){
      # For VCS 5.0
      $sDestTypeFile = MAINCFLOC . "/${sType}Types50.cf";
    }
  }
  copy ($sTypeFile, $sDestTypeFile);
  print "Old TibcoEMSTypes.cf is replaced with New TibcoEMSTypes.cf\n";
  print "For details, see the following log: [$sLogDir/${sType}_update.log]\n";
  close ( LOG );
  exit 0;
} 

 sub getVCSStatus {
  my $sCmd = HASYS . " -list -localclus";
  my @lsCmdOut = `$sCmd 2>&1`;

  $bVCSStatus = 1; # Assume vcs is currently running 

  if ( grep { /Cannot connect to VCS engine/ } @lsCmdOut ) {
    print LOG "VCS is not running. Following Offline Procedure.\n";
    $bVCSStatus = 0;
  }
  else {
    print LOG "VCS is running. Following Online Procedure.\n";
  }

  return $bVCSStatus;
 }
 
 
 sub printError {
   my ( $sErrorString ) = @_;
   warn "$sErrorString\n";
   print LOG "[Failure] $sErrorString\n";
   print "For more details check the log file [$sLogDir/${sType}_update.log]\n";
 }
 
 warn "For more details check the log file [$sLogDir/${sType}_update.log] $sRestoreMsg\n";
 exit 1;
